.include "TN15DEF.INC"					; include chip definition
;------------------------------
.def	temp1=r16
.def	temp2=r17

.def	SelectSource=r18
.def	BackToSource=r19
.def	SelectWorkFlag=r20
.def	HoldRearOnCounter=r21
.def	HoldRearOffCounter=r22

.equ	HoldRearOnDelayInitVal=3			; .17 * 3 = .51 sec
.equ	HoldRearOffDelayInitVal=40			; .17 * 40 = 6.8 sec

.equ	VGA=PINB0
.equ	FRONT=PINB1
.equ	REAR=PINB2
.equ	BUTTON=PINB4
.equ	GEAR=PINB3

.equ	bsTV=0
.equ	bsVGA=1
.equ	bsFrontCamera=2
.equ	bsRearCamera=4

;------------------------------
	rjmp	RESET					; Reset Handle
	rjmp	NOVECT					; External Interrupt0
	rjmp	NOVECT					; Pin change Interrupt
	rjmp	NOVECT					; Timer/Counter1 Compare Match A
	rjmp	NOVECT					; Overflow1 Interrupt
	rjmp	OVF0VECT				; Overflow0 Interrupt
	rjmp	NOVECT					; EEPROM ready Interrupt
	rjmp	NOVECT					; Analog Comparator Interrupt
	rjmp	NOVECT					; ADC conversion complete
;------------------------------
RESET:
; init timer
	ldi     temp1, (1<<CS02)|(1<<CS00)		; enable timer, CLK/1024
	out     TCCR0, temp1
	ldi     temp1, (1<<TOIE0)			; enable Timer0 Overflow int
	out     TIMSK, temp1
; init ports
	ldi	temp1, (1<<PINB4)			; 4 - output pin (source select button)
	out	DDRB, temp1
; clear values
	ldi	SelectSource, 0
	ldi	BackToSource, 0
	ldi	SelectWorkFlag, 0
	ldi	HoldRearOnCounter, HoldRearOnDelayInitVal
	ldi	HoldRearOffCounter, HoldRearOffDelayInitVal
; enable interrupts
	sei
;------------------------------
MAIN:
	rjmp	MAIN
;------------------------------
OVF0VECT:
	in	temp1, SREG				; Store the Status register in reg temp1
; ATTiny15 internal RC clock 1.6MHz => 1600000 / 1024 = 1562,5 / 256 = 6 => 1 / 6 = .17 sec

	tst	SelectWorkFlag
	brne	DOSELECTSOURCE


	sbis	PORTB, GEAR
	rjmp	DRIVESEL
	rjmp	REARSEL

DRIVESEL:						; P, D, N, 2, L gear selector position
	sbic	PORTB, REAR
	rjmp	D_REARCAM
	sbic	PORTB, FRONT
	rjmp	D_FRONTCAM
	sbic	PORTB, VGA
	rjmp	D_VGA
	rjmp	D_TV

REARSEL:						; R gear selector position
	sbic	PORTB, REAR
	rjmp	R_REARCAM
	sbic	PORTB, FRONT
	rjmp	R_FRONTCAM
	sbic	PORTB, VGA
	rjmp	R_VGA
	rjmp	R_TV


D_REARCAM:
	dec	HoldRearOffCounter
	brne	EX_OVF0VECT
	mov	SelectSource, BackToSource
	ldi	SelectWorkFlag, 1
	rjmp	DOSELECTSOURCE
D_FRONTCAM:
	ldi	HoldRearOnCounter, HoldRearOnDelayInitVal
	ldi	HoldRearOffCounter, HoldRearOffDelayInitVal
	rjmp	EX_OVF0VECT
D_VGA:
	ldi	HoldRearOnCounter, HoldRearOnDelayInitVal
	ldi	HoldRearOffCounter, HoldRearOffDelayInitVal
	rjmp	EX_OVF0VECT
D_TV:
	ldi	HoldRearOnCounter, HoldRearOnDelayInitVal
	ldi	HoldRearOffCounter, HoldRearOffDelayInitVal
	rjmp	EX_OVF0VECT

R_REARCAM:
	ldi	HoldRearOffCounter, HoldRearOffDelayInitVal
	rjmp	EX_OVF0VECT
R_FRONTCAM:
	dec	HoldRearOnCounter
	brne	EX_OVF0VECT
	ldi	BackToSource, bsFrontCamera
	ldi	SelectSource, bsRearCamera
	ldi	SelectWorkFlag, 1
	rjmp	DOSELECTSOURCE
R_VGA:
	dec	HoldRearOnCounter
	brne	EX_OVF0VECT
	ldi	BackToSource, bsVGA
	ldi	SelectSource, bsRearCamera
	ldi	SelectWorkFlag, 1
	rjmp	DOSELECTSOURCE
R_TV:
	dec	HoldRearOnCounter
	brne	EX_OVF0VECT
	ldi	BackToSource, bsTV
	ldi	SelectSource, bsRearCamera
	ldi	SelectWorkFlag, 1
	rjmp	DOSELECTSOURCE


DOSELECTSOURCE:
	in	temp2, PORTB
	andi	temp2, 0b00000111
	cp	temp2, SelectSource
	breq	SELECTCOMPLETE

	sbis	PORTB, BUTTON
	rjmp	PSHBTN
	rjmp	RLSBTN
PSHBTN:
	sbi	PORTB, BUTTON
	rjmp	EX_OVF0VECT
RLSBTN:
	cbi	PORTB, BUTTON
	rjmp	EX_OVF0VECT

SELECTCOMPLETE:
	cbi	PORTB, BUTTON
	ldi	SelectWorkFlag, 0

EX_OVF0VECT:
	out	SREG, temp1				; Restore Status register
	reti

;------------------------------
NOVECT:
	reti